import shlex
import sys
from subprocess import PIPE, Popen
from typing import List


class Executer:
    SUCCESS = 0
    ERROR = 1

    @staticmethod
    def run(command: str) -> None:
        p = Popen(shlex.split(command))
        print(f"-> {command}")
        p.communicate()
        if p.returncode == Executer.ERROR:
            sys.exit(Executer.ERROR)

    @staticmethod
    def run_pipe(command: str) -> List[str]:
        p = Popen(shlex.split(command), stdout=PIPE)
        print(f"-> {command}")
        result = p.communicate()
        if p.returncode == Executer.ERROR:
            sys.exit(Executer.ERROR)

        return_lines = result[0].decode("utf-8").split("\n")
        return return_lines

    @staticmethod
    def exit_error():
        sys.exit(Executer.ERROR)



from prompt_toolkit import PromptSession
from prompt_toolkit.completion import FuzzyWordCompleter
from prompt_toolkit.history import FileHistory

from awslogs_watch.lib.path import AWSLogsWatchPath
from awslogs_watch.lib.profile_config import ProfileConfig
from awslogs_watch.model import AWSLogsCommand, AWSLogsOption


class Prompt:
    COMMAND_HISTORY_NAME = "command_history"
    PROFILE_HISTORY_NAME = "profile_history"

    AWS_CRED_PATH = "~/.aws/credentials"

    def __init__(self, is_recent_history=False):
        self.is_recent_history = is_recent_history
        self.alw_path = AWSLogsWatchPath()

    def input_group(self, groups, history_path) -> str:
        completer = FuzzyWordCompleter(groups, WORD=True)
        history = FileHistory(history_path)
        session = PromptSession(history=history)
        default = self.find_recent_history(history)

        group_name = session.prompt(
            "Group  : ",
            completer=completer,
            complete_while_typing=True,
            default=default,
        )

        if group_name not in groups:
            return ""

        return group_name

    def input_command(self) -> str:
        commands = [command.name for command in list(AWSLogsCommand)]
        completer = FuzzyWordCompleter(commands)
        history = FileHistory(self.alw_path.create_filepath(self.COMMAND_HISTORY_NAME))
        session = PromptSession(history=history)
        default = self.find_recent_history(history)

        command_str = session.prompt(
            "Command: ",
            completer=completer,
            complete_while_typing=True,
            default=default,
        )

        if command_str not in commands:
            return ""

        return command_str

    def input_option(self, history_path, default="") -> str:
        options = [option.value for option in list(AWSLogsOption)]
        completer = FuzzyWordCompleter(options, WORD=True)

        history = FileHistory(history_path)
        session = PromptSession(history=history)

        default = self.find_default_from_history(default, history)
        option_str = session.prompt(
            "Option : ",
            completer=completer,
            complete_while_typing=True,
            default=default,
        )

        return option_str

    def input_profile(self, default="") -> str:
        profiles = ProfileConfig.load_profiles(self.AWS_CRED_PATH)
        completer = FuzzyWordCompleter(profiles, WORD=True)
        history = FileHistory(self.alw_path.create_filepath(self.PROFILE_HISTORY_NAME))

        session = PromptSession(history=history)
        default = self.find_default_from_history(default, history)

        profile = session.prompt(
            "Profile: ",
            completer=completer,
            complete_while_typing=True,
            default=default,
        )

        if profile not in profiles:
            return ""

        return profile

    def find_default_from_history(self, default, history) -> str:
        default = self.find_recent_history(history) or default

        return default

    def find_recent_history(self, history: FileHistory) -> str:
        if not self.is_recent_history:
            return ""

        recent_history = history.load_history_strings()
        try:
            recent_value = next(recent_history)
        except StopIteration:
            recent_value = ""

        return str(recent_value)



from enum import Enum, auto


class AWSLogsCommand(Enum):
    update = auto()
    get = auto()
    tail = auto()

    def is_update(self):
        if self == AWSLogsCommand.update:
            return True

        return False

    def is_get(self):
        if self == AWSLogsCommand.get:
            return True

        return False

    def is_tail(self):
        if self == AWSLogsCommand.tail:
            return True

        return False


class AWSLogsOption(Enum):
    FILTER_PATTERN = "--filter-pattern"
    WATCH_INTERVAL = "--watch-interval"
    NO_GROUP = "--no-group"
    NO_STREAM = "--no-stream"
    TIMESTAMP = "--timestamp"
    INGESTION_TIME = "--ingestion-time"
    START = "--start"
    END = "--end"
    COLOR = "--color"
    QUERY = "--query"



import sys
from pathlib import Path
from typing import Optional

from awslogs_watch.lib.execute import Executer
from awslogs_watch.lib.path import AWSLogsWatchPath


class AWSLogsExecutor:
    GROUP_CACHE_NAME = "groups-{profile}.cache"
    HISTORY_CACHE_FILENAME = "history-{profile}.cache"

    def __init__(
        self, profile: str = "default", *, option: Optional[str] = None
    ) -> None:
        self.profile = profile
        self.option = option
        self.path = AWSLogsWatchPath()

    def create_command(self, cmd_str):
        cmd = f"awslogs {cmd_str} --profile {self.profile}"
        if self.option:
            cmd += f" {self.option}"

        return cmd

    def load_groups(self):
        cmd = self.create_command("groups")
        groups = Executer.run_pipe(cmd)
        groups = [group for group in groups if group]

        return groups

    def load_groups_from_cache(self):
        if not Path(self.cache_path).is_file():
            print("ERROR: Please --update")
            sys.exit(1)

        with open(self.cache_path, "r") as f:
            text = f.read()

        return text.split("\n")

    def cache_groups(self):
        groups = self.load_groups()
        with open(self.cache_path, "w") as f:
            f.write("\n".join(groups))

    @property
    def cache_path(self) -> str:
        cache_filename = self.GROUP_CACHE_NAME.format(profile=self.profile)
        cache_path = self.path.create_filepath(cache_filename)
        return cache_path

    @property
    def history_path(self) -> str:
        history_filename = self.HISTORY_CACHE_FILENAME.format(profile=self.profile)
        history_path = self.path.create_filepath(history_filename)
        return history_path


class AWSLogsWatch:
    def __init__(self, profile="default"):
        self.awslogs = AWSLogsExecutor(profile)
        self.path = AWSLogsWatchPath()

    def tail(self, group):
        cmd = self.awslogs.create_command(f"get {group} --watch")
        Executer.run(cmd)

    def get(self, group):
        cmd = self.awslogs.create_command(f"get {group}")
        Executer.run(cmd)

    def update_groups(self):
        print("Updating...")
        self.awslogs.cache_groups()
        print("Updated!")

    def load_groups(self):
        groups = self.awslogs.load_groups_from_cache()

        return groups



import os
from argparse import ArgumentParser

from awslogs_watch._version import __version__
from awslogs_watch.base import AWSLogsWatch
from awslogs_watch.exceptions import AWSLogsWatchException
from awslogs_watch.lib.execute import Executer
from awslogs_watch.lib.path import AWSLogsWatchPath
from awslogs_watch.lib.prompt import Prompt
from awslogs_watch.model import AWSLogsCommand


class AWSLogsWatchConsole:
    OPTION_CACHE_NAME = "options"

    def __init__(self):
        parser = ArgumentParser(
            description="awslogs wrapper command.", usage="%(prog)s [options]"
        )
        parser.add_argument(
            "-v", "--version", action="version", version=f"%(prog)s v{__version__}"
        )
        parser.add_argument(
            "--profile", type=str, default="", help="AWS Account profile"
        )
        parser.add_argument("--option", type=str, default="", help="awslogs option")
        parser.add_argument(
            "-i", "--interactive", action="store_true", help="interactive mode"
        )
        parser.add_argument(
            "-r",
            "--recent_default",
            action="store_true",
            help="default value is recent history",
        )
        parser.add_argument("--update", action="store_true", help="Update group names")
        parser.add_argument("--tail", action="store_true", help="Tail log")
        parser.add_argument("--get", action="store_true", help="Get log")
        self.parser = parser
        self.parse_args = parser.parse_args()

        self.prompt = Prompt(is_recent_history=self.parse_args.recent_default)

    def run(self):
        profile = self.parse_args.profile or os.environ.get("AWS_PROFILE", "default")
        profile = self.load_profile(profile, self.parse_args.interactive)
        awslogs_watch = AWSLogsWatch(profile=profile)
        command = self.load_command()

        if command.is_update():
            awslogs_watch.update_groups()
            return

        awslogs_watch.awslogs.option = self.load_option(
            self.parse_args.option, self.parse_args.interactive
        )
        group = self.load_group(awslogs_watch)
        if command.is_get():
            awslogs_watch.get(group)

        if command.is_tail():
            awslogs_watch.tail(group)

    def load_command(self):
        command = ""
        if self.parse_args.update:
            command = AWSLogsCommand.update
        elif self.parse_args.get:
            command = AWSLogsCommand.get
        elif self.parse_args.tail:
            command = AWSLogsCommand.tail

        if not command:
            command_str = self.prompt.input_command()
            if not command_str:
                raise AWSLogsWatchException(f"No such command.")
            command = AWSLogsCommand[command_str]

        return command

    def load_option(self, option, is_interactive=False):
        if not is_interactive:
            return option

        option_history_path = AWSLogsWatchPath().create_filepath(self.OPTION_CACHE_NAME)
        option = self.prompt.input_option(option_history_path, default=option)

        return option

    def load_profile(self, profile, is_interactive=False):
        if not is_interactive:
            return profile

        _profile = self.prompt.input_profile(default=profile)
        if not _profile:
            raise AWSLogsWatchException(f"Please select correct profile.")

        return _profile

    def load_group(self, awslogs_watch):
        groups = awslogs_watch.load_groups()
        history_path = awslogs_watch.awslogs.history_path
        group_name = self.prompt.input_group(groups, history_path)

        if not group_name:
            raise AWSLogsWatchException(f"No such group.")

        return group_name


def start_console():
    try:
        AWSLogsWatchConsole().run()
    except KeyboardInterrupt:
        print("Quit awslogs-watch")
    except AWSLogsWatchException as e:
        print(e)
        Executer.exit_error()



from pathlib import Path
from typing import Optional


class AWSLogsWatchPath:
    CACHE_PATH = "~/.cache/awslogs_watch"

    def __init__(self, cache_path: Optional[str] = None) -> None:
        self.cache_dirpath: Path = Path(cache_path or self.CACHE_PATH).expanduser()

    def _create_dirpath(self, cache_dirpath: Path) -> None:
        if cache_dirpath.is_dir():
            return

        cache_dirpath.mkdir(parents=True, exist_ok=True)

    def create_filepath(self, filename) -> str:
        self._create_dirpath(self.cache_dirpath)
        return str(self.cache_dirpath / filename)



import codecs
from configparser import ConfigParser
from pathlib import Path
from typing import List


class ProfileConfig:
    @staticmethod
    def load_profiles(path: str) -> List[str]:
        _path = Path(path).expanduser()
        if not _path.is_file():
            return []

        config = ConfigParser()
        config.read_file(codecs.open(str(_path), "r", "utf-8"))

        return config.sections()



from setuptools import find_packages, setup

requires = ["prompt_toolkit==2.0.9", "awslogs"]
dev_requires = ["pytest", "pytest-sugar", "pytest-mock", "black"]

with open("README.md", "r", encoding="utf-8") as f:
    readme = f.read()

setup(
    name="awslogs_watch",
    version="0.2.0",
    description="awslogs-watch is a CLI tool that makes it easy to use awslogs",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="deresmos",
    author_email="deresmos@gmail.com",
    url="https://github.com/deresmos/awslogs-watch",
    python_requires=">=3.6",
    packages=find_packages(),
    include_package_data=False,
    keywords=["tools"],
    license="MIT License",
    install_requires=requires,
    extras_require={"develop": dev_requires},
    entry_points={
        "console_scripts": ["awslogs-watch = awslogs_watch.console:start_console"]
    },
)
